from typing import List as TList

import discord
import requests
from cpl_core.database.context import DatabaseContextABC
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord import app_commands
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_abc import ClientUtilsABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.helper.command_checks import CommandChecks
from bot_core.logging.command_logger import CommandLogger
from bot_data.abc.game_server_repository_abc import GameServerRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_game_ident_repository_abc import UserGameIdentRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.user_game_ident import UserGameIdent


class RegisterGroup(DiscordCommandABC):
    def __init__(
        self,
        logger: CommandLogger,
        message_service: MessageServiceABC,
        bot: DiscordBotServiceABC,
        client_utils: ClientUtilsABC,
        servers: ServerRepositoryABC,
        users: UserRepositoryABC,
        game_server: GameServerRepositoryABC,
        user_game_ident: UserGameIdentRepositoryABC,
        db: DatabaseContextABC,
        t: TranslatePipe,
    ):
        DiscordCommandABC.__init__(self)

        self._logger = logger
        self._message_service = message_service
        self._bot = bot
        self._client_utils = client_utils
        self._servers = servers
        self._users = users
        self._game_server = game_server
        self._user_game_ident = user_game_ident
        self._db = db
        self._t = t

        self._logger.trace(__name__, f"Loaded command service: {type(self).__name__}")

    @commands.hybrid_group()
    @commands.guild_only()
    async def register(self, ctx: Context):
        pass

    @register.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def minecraft(self, ctx: Context, member: discord.Member, game_server: int, name: str):
        self._logger.debug(__name__, f"Received command register minecraft {ctx}")

        minecraft_id = None
        try:
            self._logger.debug(__name__, f"Try to get minecraft id for {name}")
            response = requests.get(url=f"https://api.mojang.com/users/profiles/minecraft/{name}")
            if len(response.content) == 0:
                await self._message_service.send_interaction_msg(
                    ctx.interaction, self._t.transform("modules.base.register.not_found")
                )
                return

            minecraft_id = response.json()["id"]
        except Exception as e:
            self._logger.error(__name__, f"Get minecraft id for {name} failed", e)
            await self._message_service.send_interaction_msg(
                ctx.interaction, self._t.transform("modules.base.register.not_found")
            )

        if minecraft_id is None:
            return

        server = self._servers.get_server_by_discord_id(ctx.guild.id)
        user = self._users.get_user_by_discord_id_and_server_id(member.id, server.id)
        gs = self._game_server.get_game_server_by_id(game_server)

        game_ident = UserGameIdent(user, gs, minecraft_id)
        self._user_game_ident.add_user_game_ident(game_ident)
        self._db.save_changes()

        await self._message_service.send_interaction_msg(
            ctx.interaction, self._t.transform("modules.base.register.success")
        )

        self._logger.trace(__name__, f"Finished register minecraft command")

    @minecraft.autocomplete("game_server")
    async def game_server_autocomplete(
        self, interaction: discord.Interaction, current: str
    ) -> TList[app_commands.Choice[str]]:
        server = self._servers.get_server_by_discord_id(interaction.guild.id)
        game_servers = self._game_server.get_game_servers_by_server_id(server.id)

        return [
            app_commands.Choice(name=gs.name, value=gs.id)
            for gs in self._client_utils.get_auto_complete_list(game_servers, current, lambda x: x.name)
        ]
